/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2010, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.client.api;
 
import gov.lbl.srm.StorageResourceManager.*;
import gov.lbl.srm.client.wsdl.*;

import java.io.*;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.log4j.PropertyConfigurator;

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMCopyTest
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public class SRMCopyTest {

   public static void main(String[] args) {
     String serverUrl = "";
     String uid="";
     String logPath="";
     String log4jlocation="";
     String storageInfo="";
     String targetStorageInfo="";
     String fileType="volatile";
     String retentionPolicy="replica";
     String accessLatency="online";
     boolean debug = false;
     boolean silent = false;
     boolean delegationNeeded=false;
     Vector vec = new Vector ();
     Vector tvec = new Vector ();

     String ttemp = System.getProperty("log4j.configuration");
     if(ttemp != null && !ttemp.equals("")) {
        log4jlocation = ttemp;
     }

     for(int  i = 0; i < args.length; i++) {
       System.out.println("ARGS[i]="+args[i]);
       if(args[i].equals("-serviceurl") && i+1 < args.length) {
          serverUrl = args[i+1];
          i++;
       }
       else if(args[i].equals("-logpath") && i+1 < args.length) {
          logPath = args[i+1];
          i++;
       }
       else if(args[i].equals("-uid") && i+1 < args.length) {
          uid = args[i+1];
          i++;
       }
       else if(args[i].equals("-storageinfo") && i+1 < args.length) {
          storageInfo = args[i+1];
          i++;
       }
       else if(args[i].equals("-filetype") && i+1 < args.length) {
          fileType = args[i+1];
          i++;
       }
       else if(args[i].equals("-retentionpolicy") && i+1 < args.length) {
          retentionPolicy = args[i+1];
          i++;
       }
       else if(args[i].equals("-accesslatency") && i+1 < args.length) {
          accessLatency = args[i+1];
          i++;
       }
       else if(args[i].equals("-targetstorageinfo") && i+1 < args.length) {
          targetStorageInfo = args[i+1];
          i++;
       }
       else if(args[i].equals("-surls") && i+1 < args.length) {
          String temp = args[i+1];
          StringTokenizer ss = new StringTokenizer(temp,";");
          while (ss.hasMoreTokens()) {
            vec.addElement(ss.nextToken());
          }
          i++;
       }
       else if(args[i].equals("-turls") && i+1 < args.length) {
          String temp = args[i+1];
          StringTokenizer ss = new StringTokenizer(temp,";");
          while (ss.hasMoreTokens()) {
            tvec.addElement(ss.nextToken());
          }
          i++;
       }
       else if(args[i].equals("-debug")) {
          debug=true;
       }
       else if(args[i].equals("-delegation")) {
          delegationNeeded=true;
       }
       else if(args[i].equals("-silent")) {
          silent=true;
       }
       else if(args[i].equals("-help")) {
          showUsage();
       }
     }

    delegationNeeded=true;
    System.out.println("ServerUrl="+serverUrl);
    System.out.println("uid="+uid);
    System.out.println("RequestType=copy");
    System.out.println("debug="+debug);
    System.out.println("delegation="+delegationNeeded);
    System.out.println("silent="+silent);
    System.out.println("FileStorageType="+fileType);
    System.out.println("RetentionPolicy="+retentionPolicy);
    System.out.println("AccessLatency="+accessLatency);
    if(storageInfo.equals("")) {
      System.out.println("storageInfo="+storageInfo);
    }
    if(targetStorageInfo.equals("")) {
      System.out.println("targetStorageInfo="+targetStorageInfo);
    }

    String[] surl = new String[vec.size()];
    for(int i = 0; i < vec.size(); i++) {
        System.out.println("SURL["+i+"]="+vec.elementAt(i));
        surl[i] = (String)vec.elementAt(i);
    }

    String[] turl = new String[tvec.size()];
    for(int i = 0; i < tvec.size(); i++) {
        System.out.println("TURL["+i+"]="+tvec.elementAt(i));
        turl[i] = (String)tvec.elementAt(i);
    }

    if(serverUrl.equals("")) {
      System.out.println("Cannot connect to service with null serviceurl");
      System.exit(1);
    }

    if(surl == null || surl.length == 0) {
      System.out.println("Please provide the surls");
      System.exit(1);
    }

    if(turl == null || turl.length == 0) {
      System.out.println("Please provide the turls");
      System.exit(1);
    }
      
    try {
      if(!storageInfo.equals("")) {
        delegationNeeded=true;
      }
      SRMServer cc = new SRMServer
			(log4jlocation, logPath,debug,delegationNeeded);
      cc.connect(serverUrl);
      cc.ping(uid);
      SRMRequest req = new SRMRequest();
      req.setSRMServer(cc);
      req.setAuthID(uid);
      req.setRequestType("copy");
      req.addFiles(surl, turl,null);
      req.setStorageSystemInfo(storageInfo);
      req.setTargetStorageSystemInfo(targetStorageInfo);
      req.setFileStorageType(fileType);
      req.setRetentionPolicy(retentionPolicy);
      req.setAccessLatency(accessLatency);
      req.submit();
      req.checkStatus();

      SRMRequestStatus response = req.getStatus();
      req.printResults();
      cc.disconnect();
    }catch(Exception e) {
      e.printStackTrace();
    }
   }


   public static void showUsage() {
       System.out.println("............................................");
       System.out.println("java gov.lbl.srm.client.api.SRMCopyTest ");
       System.out.println("-serviceurl      <endpoint>");
       System.out.println("-uid      <authid>");
       System.out.println("-logpath      <path to logfile>");
       System.out.println("-filetype      <volative|permanent|durable>(default:volatile)");
       System.out.println("-retentionpolicy <replica|output|custodial>(default:replica)");
       System.out.println("-accesslatency <online|nearline>(default:online)");
       System.out.println("-surls      <surls are given delimiter is \";\">");
       System.out.println("\t srm://dmx.lbl.gov:6250/srm/v2/server?SFN=/srmcache/~/put.test;"+
				              "srm://dmx.lbl.gov:6250/srm/v2/server?SFN=/srmcache/~/put.test.1");
       System.out.println("-turls      <turls are given delimiter is \";\">");
       System.out.println("\t srm://dmx.lbl.gov:6250/srm/v2/server?SFN=/srmcache/~/copy.test;"+
				              "srm://dmx.lbl.gov:6250/srm/v2/server?SFN=/srmcache/~/copy.test.1");
       System.out.println("-debug  (true|fase) (default:false)");
       System.out.println("-silent (true|false) (default:false)");
       System.out.println("-help  (to show this message)");
       System.exit(1);
     }
}

