/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2010, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.client.api;
 
import gov.lbl.srm.StorageResourceManager.*;
import gov.lbl.srm.client.wsdl.*;

import java.io.*;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.log4j.PropertyConfigurator;

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMPutConstruct
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public class SRMPutConstruct {

   public static void main(String[] args) {
     String serverUrl = "";
     String logPath = "";
     String log4jlocation="";
     String requestToken="";
     boolean debug = false;
     boolean silent = false;
     boolean delegationNeeded=false;
     Vector vec = new Vector ();

     String ttemp = System.getProperty("log4j.configuration");
     if(ttemp != null && !ttemp.equals("")) {
        log4jlocation = ttemp;
     }

     for(int  i = 0; i < args.length; i++) {
       System.out.println("ARGS[i]="+args[i]);
       if(args[i].equals("-serviceurl") && i+1 < args.length) {
          serverUrl = args[i+1];
          i++;
       }
       else if(args[i].equals("-logpath") && i+1 < args.length) {
          logPath = args[i+1];
          i++;
       }
       else if(args[i].equals("-rid") && i+1 < args.length) {
          requestToken = args[i+1];
          i++;
       }
       else if(args[i].equals("-debug")) {
          debug=true;
       }
       else if(args[i].equals("-delegation")) {
          delegationNeeded=true;
       }
       else if(args[i].equals("-silent")) {
          silent=true;
       }
       else if(args[i].equals("-help")) {
          showUsage();
       }
     }

    System.out.println("ServerUrl="+serverUrl);
    System.out.println("RequestType=put");
    System.out.println("RequestToken="+requestToken);
    System.out.println("debug="+debug);
    System.out.println("delegation="+delegationNeeded);
    System.out.println("silent="+silent);

    if(serverUrl.equals("")) {
      System.out.println("Cannot connect to service with null serviceurl");
      System.exit(1);
    }

    try {
      SRMServer cc = new SRMServer
			(log4jlocation, logPath, debug,delegationNeeded);
      cc.connect(serverUrl);
      cc.ping("viji");
      SRMRequest req = new SRMRequest();
      req.setSRMServer(cc);
      req.setRequestType("put");
      req.setRequestToken(requestToken);
      System.out.println("Calling checkStatus");
      req.checkStatus();

      System.out.println("Calling getStatus");
      SRMRequestStatus response = req.getStatus();
      req.printResults();
      System.out.println("Calling getFiles");
      String[] str = req.getFiles();
      for(int i = 0; i < str.length; i++) {
        System.out.println("Files["+i+"]="+str[i]);
      }
      cc.disconnect();
    }catch(Exception e) {
      e.printStackTrace();
    }
   }


   public static void showUsage() {
       System.out.println("............................................");
       System.out.println("java gov.lbl.srm.client.api.SRMPutTest ");
       System.out.println("-serviceurl      <endpoint>");
       System.out.println("-uid      <authid>");
       System.out.println("-logpath      <path to logfile>");
       System.out.println("-rid      <request token returned by SRM server>");
       System.out.println("-debug  (true|fase) (default:false)");
       System.out.println("-silent (true|false) (default:false)");
       System.out.println("-help  (to show this message)");
       System.exit(1);
     }
}

