/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2010, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.client.wsdl;
 
import gov.lbl.srm.StorageResourceManager.*;
//import gov.lbl.srm.util.*;
import gov.lbl.srm.client.api.*;
import org.globus.axis.util.Util;

import java.io.*;
import java.net.URL;
import java.util.Vector;
import java.util.HashMap;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;


import javax.xml.rpc.Stub;
import org.apache.axis.types.URI;
import org.apache.axis.types.UnsignedLong;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.*;
import org.apache.log4j.PropertyConfigurator;

import org.globus.axis.gsi.GSIConstants;
import org.gridforum.jgss.ExtendedGSSCredential;
import org.ietf.jgss.GSSCredential;
import org.gridforum.jgss.ExtendedGSSManager;

//import org.globus.ogsa.utils.GSIUtils;
import org.globus.util.ConfigUtil;
//import org.globus.ogsa.utils.GetOpts;
//import org.globus.ogsa.gui.ServiceContext;
import org.globus.gsi.GlobusCredential;

import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.gsi.gssapi.auth.*;

import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;

import gov.lbl.srm.client.exception.*;


//import org.globus.wsrf.impl.security.authentication.Constants;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMCopyClient
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

class SRMCopyClient {
  private ISRM _srm;
  private GSSCredential _credential;
  private static boolean _debug;
  private static boolean silent;
  private static java.util.logging.Logger _theLogger;
  private static Vector inputVec = new Vector ();

  private String authId = "";
  private String userDesc = ""; 
  private boolean overwrite;
  private int totalRequestTime;
  private int targetSURLLifeTime;
  private String storageInfo;
  private String targetStorageInfo;
  private TFileStorageType fileStorageType;
  private String spaceToken;
  private TRetentionPolicyInfo retentionPolicyInfo;
  private int bufferSize;
  private boolean dcau;
  private int parallelism;
  private boolean pushMode=false;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMCopyClient --- constructor
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SRMCopyClient (ISRM srm, GSSCredential credential,
	boolean debug, boolean silent, 
	java.util.logging.Logger theLogger, int bufferSize, int parallelism,
	boolean dcau) throws Exception 
{
   _srm = srm;
   _credential = credential;
   _debug = debug;
   this.silent = silent;
   _theLogger = theLogger;
   this.bufferSize = bufferSize;
   this.parallelism = parallelism;
   this.dcau = dcau;
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setPushMode
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
void setPushMode(boolean b) {
  pushMode = b;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// doCopy
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmCopyResponse doCopy(String[] surls, String[] turls) 
	throws SRMClientException, Exception {
   
  try {
   inputVec = new Vector();
   validateInput (surls,turls);
   inputVec.addElement("Validated input");
   util.printEventLog(_theLogger,"doCopy",inputVec,silent);
   SrmCopyResponse response = 
		copyFiles (surls,turls,spaceToken);
   if(_debug) {
     SRMCopyClient.printCopyResult(response);
   }
   return response;
  }catch(Exception e) {
    inputVec = new Vector ();
    inputVec.addElement("Exception="+e.getMessage());
    util.printEventLog(_theLogger,"doCopy",inputVec,silent);
    util.printStackTrace(e,_theLogger);
    throw new SRMClientException(e.getMessage());
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// validateInput
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void validateInput(String[] surl, String[] turl) throws Exception { 
  for(int i = 0; i < turl.length; i++) {
    if(surl[i].startsWith("srm:") && turl[i].startsWith("srm:")) {; }
    else {
      throw new Exception("Given Input is not valid, " +
			"turl must start with srm for put request");
    }
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setRetentionPolicyInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setRetentionPolicyInfo(TRetentionPolicyInfo retentionPolicy) {
   retentionPolicyInfo = retentionPolicy;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setFileStorageInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setFileStorageType(TFileStorageType fileStorageType) {
    this.fileStorageType= fileStorageType;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setUserDesc
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setUserDesc(String userDesc) {
  this.userDesc = userDesc;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setAuthID
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setAuthID(String authId) {
  this.authId = authId;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTargetSpaceToken
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setTargetSpaceToken(String spaceToken) {
  this.spaceToken = spaceToken;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setStorageSystemInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setStorageSystemInfo(String storageInfo) {
   this.storageInfo = storageInfo;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTargetStorageSystemInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setTargetStorageSystemInfo(String storageInfo) {
   this.targetStorageInfo = storageInfo;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setOverwrite
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setOverwrite(boolean overwrite) {
   this.overwrite = overwrite;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTotalRequestTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setTotalRequestTime(int totalRequestTime) {
   this.totalRequestTime = totalRequestTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setTargetSURLLifeTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setTargetSURLLifeTime(int fileLifeTime) {
  this.targetSURLLifeTime = fileLifeTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// copyFiles
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	
private SrmCopyResponse copyFiles (String[] surls,
	String[] turls, String token)  throws Exception {


  /*
  if(token != null) {
      if(_debug)  {
        getSpaceTokenMeta(token);
      }
   }
   */

   if(pushMode) {
     util.printMessage("\nSRM-CLIENT: " + new Date() + " Calling srmCopy now in Push Mode", silent);
   }
   else {
     util.printMessage("\nSRM-CLIENT: " + new Date() + " Calling srmCopy now in Pull Mode", silent);
   }
 
   if(_debug) {
     util.printMessage("SRM-CLIENT: ::::::::::::::Input parameters ::::::", silent);
   }

   for(int i =  0; i < surls.length; i++) {
      inputVec = new Vector();
      if(_debug) {
         util.printMessage("SRM-CLIENT: SourceSURL["+i+"]="+ surls[i],silent);
         util.printMessage("SRM-CLIENT: TargetSURL["+i+"]="+ turls[i],silent);
      }
      inputVec.addElement("SourceSURL["+i+"]="+ surls[i]);
      inputVec.addElement("TargetSURL["+i+"]="+ turls[i]);
      util.printEventLog(_theLogger,"SrmCopy",inputVec,silent);
   } 

   SrmCopyRequest r = new SrmCopyRequest();
 
   inputVec = new Vector();
   if(!userDesc.equals("")) {
     inputVec.addElement("UserDescription="+userDesc);
     util.printEventLog(_theLogger,"SrmCopy",inputVec,silent);
     if(_debug) {
       util.printMessage("SRM-CLIENT: UserDescription="+userDesc, silent);
     }
     r.setUserRequestDescription(userDesc);
   }
 
   if(!authId.equals("")) {
     r.setAuthorizationID(authId);
   }

   inputVec = new Vector();
   inputVec.addElement("AuthorizationID="+authId);
 
   if(_debug) {
     util.printMessage("SRM-CLIENT: AuthorizationID="+authId,silent);
     util.printMessage("SRM-CLIENT: UserRequestDescription=test user description",silent);
   }
   inputVec.addElement("UserRequestDescription=test user description");
   r.setUserRequestDescription("test user description");
   if(token != null && !token.equals("")) {
     r.setTargetSpaceToken(token);
     if(_debug) {
       util.printMessage("SRM-CLIENT: SpaceToken="+token, silent);
     }
     inputVec.addElement("SpaceToken="+token);
     util.printEventLog(_theLogger,"SrmCopy",inputVec,silent);
   }

   if(overwrite) {
     r.setOverwriteOption(TOverwriteMode.ALWAYS);
   }
   else {
     r.setOverwriteOption(TOverwriteMode.NEVER);
   }
 
   inputVec = new Vector();
   inputVec.addElement("Overwrite="+overwrite);
   if(_debug) {
     util.printMessage("SRM-CLIENT: Overwrite="+overwrite, silent);
   }

   if(totalRequestTime != 0) {
     r.setDesiredTotalRequestTime(new Integer(totalRequestTime));
     inputVec.addElement("TotalTime="+totalRequestTime);
     if(_debug) {
       util.printMessage("SRM-CLIENT: TotalTime="+totalRequestTime, silent);
     }
   }


   if(targetSURLLifeTime != 0) {
     r.setDesiredTargetSURLLifeTime(new Integer(targetSURLLifeTime));
     inputVec.addElement("TargetSURLLifeTime="+targetSURLLifeTime);
     if(_debug) {
       util.printMessage("SRM-CLIENT: TargetSURLLifeTime="+targetSURLLifeTime, silent);
     }
   }

   if(fileStorageType != null) {
     r.setTargetFileStorageType(fileStorageType);
     inputVec.addElement("FileStorageType="+fileStorageType);
     if(_debug) {
       util.printMessage("SRM-CLIENT: FileStorageType="+fileStorageType, silent);
     }
   }

   if(retentionPolicyInfo != null) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: RetentionPolicy="+
			retentionPolicyInfo.getRetentionPolicy(), silent);
     }
     inputVec.addElement("RetentionPolicy="+retentionPolicyInfo.getRetentionPolicy());
     if(_debug) {
        util.printMessage("SRM-CLIENT: ACCESSINFO="+
			retentionPolicyInfo.getAccessLatency(), silent);
     }
     inputVec.addElement("ACCESSINFO="+retentionPolicyInfo.getAccessLatency());
     r.setTargetFileRetentionPolicyInfo(retentionPolicyInfo);
   }

   int total = surls.length;
 
   TCopyFileRequest[] fileReqList = new TCopyFileRequest[total];
 
   ExtendedGSSCredential cred = (ExtendedGSSCredential) _credential;
   byte[] bb = cred.export(ExtendedGSSCredential.IMPEXP_OPAQUE);
   String proxyString = new String(bb);
 
   boolean useSourceStorageSystemInfo = false;
   boolean useTargetStorageSystemInfo = false; 
 
   for (int i=0; i<total; i++) {
     TCopyFileRequest curr = new TCopyFileRequest();
     curr.setSourceSURL(new URI(surls[i]));
     curr.setTargetSURL(new URI(turls[i]));
 
     int idx = surls[i].indexOf("mss://");
 
     if(storageInfo.equals("")) {
      if(idx != -1) {
         useSourceStorageSystemInfo = true;
      }
 
     idx = turls[i].indexOf("mss://");
      if(idx != -1) {
         useTargetStorageSystemInfo = true;
      }
     }
     fileReqList[i] = curr;
   }

   inputVec.addElement("SourceStorageSystemInfo="+useSourceStorageSystemInfo);
   inputVec.addElement("TargetStorageSystemInfo="+useTargetStorageSystemInfo);
   ArrayOfTCopyFileRequest arrayOfCopy = new ArrayOfTCopyFileRequest();
   arrayOfCopy.setRequestArray(fileReqList);
 
   r.setArrayOfFileRequests(arrayOfCopy);
 
   ArrayOfTExtraInfo storageInfoArray = new ArrayOfTExtraInfo();
 
   Vector vec = new Vector();
 
   TExtraInfo tExtra = new TExtraInfo();
   tExtra.setKey("gsiftpStream");
   tExtra.setValue(""+parallelism);
   vec.addElement(tExtra);
   if(_debug) {
     util.printMessage("SRM-CLIENT: gsiftpStream="+parallelism, silent);
   }
   inputVec.addElement("gsiftpStream="+parallelism);
 
   tExtra = new TExtraInfo();
   tExtra.setKey("bufferSize");
   tExtra.setValue(""+bufferSize);
   vec.addElement(tExtra);
   if(_debug) {
     util.printMessage("SRM-CLIENT: BufferSize="+bufferSize, silent);
   }
   inputVec.addElement("BufferSize="+bufferSize);

   tExtra = new TExtraInfo();
   tExtra.setKey("dcau");
   tExtra.setValue(""+dcau);
   vec.addElement(tExtra);
   if(_debug) {
     util.printMessage("SRM-CLIENT: DCAU="+dcau, silent);
   }
   inputVec.addElement("DCAU="+dcau);
   int checkIdx = storageInfo.indexOf("#");
   String sInfo2 = "";

   if(useSourceStorageSystemInfo) {
     tExtra = new TExtraInfo();
     tExtra.setKey("uid");
     tExtra.setValue("copy");
     vec.addElement(tExtra);
 
     tExtra = new TExtraInfo();
     tExtra.setKey("pwd");
     tExtra.setValue(proxyString);
     vec.addElement(tExtra);
   }
   else if(storageInfo.startsWith("for")) {
      if(checkIdx != -1) {
        sInfo2 = storageInfo.substring(checkIdx+1);
        storageInfo = storageInfo.substring(0,checkIdx);
      }
      StringTokenizer stk = new StringTokenizer(storageInfo,",");
      while(stk.hasMoreTokens()) {
         String temp = stk.nextToken();
         int idx = temp.indexOf(":");
         if(idx != -1) {
           String key = temp.substring(0,idx);
           String value = temp.substring(idx+1);
           tExtra = new TExtraInfo();
           tExtra.setKey(key);
           tExtra.setValue(value);
           vec.addElement(tExtra);
         }
         else {
           String msg = "Given storage info is not in the correct format " +
            "Please use the format for:source,login:uid,passwd:pwd" +
            " Given storage info is " + storageInfo;
           throw new Exception(msg);
         }
      }
   }

   if(checkIdx !=-1) {
     storageInfo = sInfo2;
   }
 
 
   TExtraInfo[] a_tExtra = new TExtraInfo[vec.size()];
   for(int i = 0; i < vec.size(); i++) {
     a_tExtra[i] = (TExtraInfo) vec.elementAt(i);
   }
   storageInfoArray.setExtraInfoArray(a_tExtra);
   r.setSourceStorageSystemInfo(storageInfoArray);
 
   ArrayOfTExtraInfo targetStorageInfoArray = new ArrayOfTExtraInfo();
   vec = new Vector();
 
   tExtra = new TExtraInfo();
   tExtra.setKey("gsiftpStream");
   tExtra.setValue(""+parallelism);
   vec.addElement(tExtra);
   if(_debug) {
     util.printMessage("SRM-CLIENT: gsiftpStream="+parallelism, silent);
   }
   inputVec.addElement("gsiftpStream="+parallelism);
 
   tExtra = new TExtraInfo();
   tExtra.setKey("bufferSize");
   tExtra.setValue(""+bufferSize);
   vec.addElement(tExtra);
   if(_debug) {
     util.printMessage("SRM-CLIENT: bufferSize="+bufferSize, silent);
   }
   inputVec.addElement("bufferSize="+bufferSize);
    
   tExtra = new TExtraInfo();
   tExtra.setKey("dcau");
   tExtra.setValue(""+dcau);
   vec.addElement(tExtra);
   if(_debug) {
     util.printMessage("SRM-CLIENT: DCAU="+dcau, silent);
   }
   inputVec.addElement("DCAU="+dcau);

   if(useTargetStorageSystemInfo) {
     tExtra = new TExtraInfo();
     tExtra.setKey("uid");
     tExtra.setValue("copy");
     vec.addElement(tExtra);
 
     tExtra = new TExtraInfo();
     tExtra.setKey("pwd");
     tExtra.setValue(proxyString);
     vec.addElement(tExtra);
   }
   else if(storageInfo.startsWith("for")) {
      StringTokenizer stk = new StringTokenizer(storageInfo,",");
      while(stk.hasMoreTokens()) {
         String temp = stk.nextToken();
         int idx = temp.indexOf(":");
         if(idx != -1) {
           String key = temp.substring(0,idx);
           String value = temp.substring(idx+1);
           tExtra = new TExtraInfo();
           tExtra.setKey(key);
           tExtra.setValue(value);
           vec.addElement(tExtra);
         }
         else {
           throw new Exception(
            "Given storage info is not in the correct format " + storageInfo);
         }
      }
   }

   a_tExtra = new TExtraInfo[vec.size()];
   for(int i = 0; i < vec.size(); i++) {
     a_tExtra[i] = (TExtraInfo) vec.elementAt(i);
   }
   targetStorageInfoArray.setExtraInfoArray(a_tExtra);
   r.setTargetStorageSystemInfo(targetStorageInfoArray);
 
   inputVec.addElement("TimeStamp="+new Date());
   util.printEventLog(_theLogger,"Sending SrmCopy request now",inputVec,silent);
 
   SrmCopyResponse result = _srm.srmCopy(r);
 
   return result;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// doStatus
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmStatusOfCopyRequestResponse doStatus (String requestId,
		String[] surl, String[] turl) throws Exception {

 if(_debug) {
     util.printMessage("\nSRM-CLIENT: ...Input parameters for SrmStatusOfCopyRequst...",silent);
     util.printMessage("SRM-CLIENT: RequestToken="+requestId,silent);
     util.printMessage("SRM-CLIENT: AuthId="+authId,silent);
 }
 
 SrmStatusOfCopyRequestRequest r = new SrmStatusOfCopyRequestRequest();

 int size = surl.length;
 URI[] fromsurl = new URI[size];
 URI[] tosurl  = new URI[size];

 for(int i = 0; i < size; i++) {
   fromsurl[i] =  new URI(surl[i]);
   tosurl[i] = new URI(turl[i]);
 }

 if(size > 0) {
   ArrayOfAnyURI arrayOfFromURI = SRMUtilClient.convertToArrayOfAnyURI(fromsurl);
   ArrayOfAnyURI arrayOfToURI   = SRMUtilClient.convertToArrayOfAnyURI(tosurl);
   r.setArrayOfSourceSURLs(arrayOfFromURI);
   r.setArrayOfTargetSURLs(arrayOfToURI);
 } 

 r.setRequestToken(requestId);
 if(!authId.equals("")) {
   r.setAuthorizationID(authId);
 }
 
 SrmStatusOfCopyRequestResponse result = _srm.srmStatusOfCopyRequest(r);
  
 if(_debug) {
   util.printMessage("\nSRM-CLIENT: #### Output from SRM ####",silent);
 }
 
 if(result == null) {
   inputVec = new Vector();
   inputVec.addElement("RequestId="+requestId);
   inputVec.addElement("Null result");
   inputVec.addElement("May be the given requestId is not valid ");
   if(_debug) {
     util.printEventLog(_theLogger,"SrmCopy",inputVec,silent);
     util.printMessage("\nSRM-CLIENT: Null result",silent);
     util.printMessage("SRM-CLIENT: May be the given requestId is not valid ",silent);
     util.printMessage("SRM-CLIENT: RequestId="+requestId,silent);
   }
   return result;
 }

 TReturnStatus rStatus = result.getReturnStatus();
 if(rStatus != null) {
    TStatusCode sCode = rStatus.getStatusCode();
    if(_debug) {
      util.printMessage("\nRequestStatus="+sCode.toString(),silent);
      util.printMessage("\nRequestExplanation="+rStatus.getExplanation(),silent);
    }
    inputVec = new Vector();
    inputVec.addElement("RequestId="+requestId);
    inputVec.addElement("RequestStatus="+sCode.toString());
    inputVec.addElement("RequestExplanation="+rStatus.getExplanation());
    util.printEventLog(_theLogger,"SrmCopy",inputVec,silent);
 }
 else {
    if(_debug) {
      util.printMessage("Null status code for this request", silent);
    } 
    inputVec = new Vector();
    inputVec.addElement("RequestId="+requestId);
    inputVec.addElement("null status code for this request");
    util.printEventLog(_theLogger,"SrmCopy",inputVec,silent);
    return result;
 }

 ArrayOfTCopyRequestFileStatus arrayFStatus = result.getArrayOfFileStatuses();

 if(arrayFStatus == null) {
    if(_debug) {
      util.printMessage("Null getArrayOfFileStatuses", silent);
    } 
    inputVec = new Vector();
    inputVec.addElement("RequestId="+requestId);
    inputVec.addElement("null getArrayOfFileStatuses");
    util.printEventLog(_theLogger,"SrmCopy",inputVec,silent);
    return result;
 }
 
 TCopyRequestFileStatus[] fileStatus = arrayFStatus.getStatusArray();
 
 if(fileStatus.length == 0) {
   inputVec = new Vector();
   inputVec.addElement("No files in this request");
   util.printEventLog(_theLogger,"SrmCopy",inputVec,silent);
   if(_debug) {
     util.printMessage("SRM-CLIENT: No files in this request", silent);
   }
   return result;
 }

 for(int i = 0; i < fileStatus.length; i++) {
   inputVec = new Vector();
   if(_debug) {
     util.printMessage("\nSRM-CLIENT: ...........................................",silent);
   }
   if(fileStatus[i].getSourceSURL() != null) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: SourceSURL="+
          fileStatus[i].getSourceSURL().toString(),silent);
     }
     inputVec.addElement("SourceSURL="+fileStatus[i].getSourceSURL().toString());
   }
   if(fileStatus[i].getTargetSURL() != null) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: TargetSURL="+
          fileStatus[i].getTargetSURL().toString(),silent);
     }
     inputVec.addElement("TargetSURL="+fileStatus[i].getTargetSURL().toString());
   }
   if(fileStatus[i].getStatus() != null) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: Status="+
         fileStatus[i].getStatus().getStatusCode(),silent);
       util.printMessage("SRM-CLIENT: Explanation="+
         fileStatus[i].getStatus().getExplanation(),silent);
     }
     inputVec.addElement("Status="+fileStatus[i].getStatus().getStatusCode());
     inputVec.addElement("Explanation="+fileStatus[i].getStatus().getExplanation());
   }
   if(fileStatus[i].getFileSize() != null) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: FileSize="+
          fileStatus[i].getFileSize(),silent);
     }
     inputVec.addElement("FileSize="+fileStatus[i].getFileSize());
   }
   if(fileStatus[i].getRemainingFileLifetime() != null) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: RemainingFileLifeTime="+
         fileStatus[i].getRemainingFileLifetime(),silent);
     }
     inputVec.addElement("RemainingFileLifeTime="+fileStatus[i].getRemainingFileLifetime());
   }

    if(fileStatus[i].getEstimatedWaitTime() != null) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: EstimatedWaitTime="+
          fileStatus[i].getEstimatedWaitTime(),silent);
     }
     inputVec.addElement("EstiamtedWaitTime="+fileStatus[i].getEstimatedWaitTime());
   }
 }
 util.printEventLog(_theLogger,"SrmCopyStatus",inputVec,silent);
 return result;
}



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// printCopyResult
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

static void printCopyResult (SrmCopyResponse response) throws Exception {

   if(_debug) {
      util.printMessage("\nSRM-CLIENT: #### Output from SRM ####", silent);
   }
 
   if(response == null) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: CopyRequest Response is null",silent);
     }
     inputVec = new Vector();
     inputVec.addElement("CopyRequest response is null");
     util.printEventLog(_theLogger,"SrmCopyResult",inputVec,silent);
     return;
   }

   inputVec = new Vector();
   if(response.getRequestToken() != null) {
     inputVec.addElement("request.token="+response.getRequestToken().toString());
     if(_debug) {
       util.printMessage("\trequest.token="+
        response.getRequestToken().toString(),silent);
     }
   }
   else {
     if(_debug) {
       util.printMessage("\trequest.token=null",silent);
     }
     inputVec.addElement("request.token=null");
   }
 
   util.printEventLog(_theLogger,"SrmCopyResult",inputVec,silent);

   if(response.getReturnStatus().getStatusCode() == null) {
      inputVec = new Vector();
      inputVec.addElement("Null ReturnStatus returned by SRM in SrmCopyRequest ");
      util.printEventLog(_theLogger,"SrmCopyResult",inputVec,silent);
      if(_debug) {
        util.printMessage("SRM-CLIENT: Null ReturnStatus returned by SRM in SrmCopyRequest",silent);
      }
      return; 
   }

   inputVec = new Vector();
   inputVec.addElement("status=" + response.getReturnStatus().getStatusCode());
 
   util.printMessage("\tstatus=" +
        response.getReturnStatus().getStatusCode(),silent);
 
   String explanation = response.getReturnStatus().getExplanation();
   if(explanation != null) {
     util.printMessage("\trequest.explanation=" + explanation,silent);
     inputVec.addElement("request.explanation=" + explanation);
   }
   util.printEventLog(_theLogger,"SrmCopyResult",inputVec,silent);

   if(response.getArrayOfFileStatuses() == null) {
      inputVec = new Vector();
      inputVec.addElement("SRM returns null ArrayOfFileStatus from SrmCopyRequest");
      util.printEventLog(_theLogger,"SrmCopyResult",inputVec,silent);
      if(_debug) {
         util.printMessage(
			"SRM-CLIENT: SRM returns null ArrayOfFileStatus from SrmCopyRequest", silent);
      }
      return; 
   }
 
   if(response.getArrayOfFileStatuses().getStatusArray() == null) {
      inputVec = new Vector();
      inputVec.addElement("SRM returns null StatusArray from SrmCopyRequest");
      util.printEventLog(_theLogger,"SrmCopyResult",inputVec,silent);
      if(_debug) {
         util.printMessage(
			"SRM-CLIENT: SRM returns null StatusArray from SrmCopyRequest", silent);
      }
      return;
   }
 
   int size = response.getArrayOfFileStatuses().getStatusArray().length;
 
   for (int i=0;  i < size; i++) {
     TCopyRequestFileStatus fileStatus =
        response.getArrayOfFileStatuses().getStatusArray(i);
     inputVec = new Vector();
     inputVec.addElement("SourceSURL="+fileStatus.getSourceSURL().toString());
     inputVec.addElement("TargetSURL="+fileStatus.getTargetSURL().toString());
     inputVec.addElement("Status="+fileStatus.getStatus().getStatusCode());
     inputVec.addElement("Explanation="+fileStatus.getStatus().getExplanation());
     util.printEventLog(_theLogger,"SrmCopyResult",inputVec,silent);
     if(_debug) {
       util.printMessage("SourceSURL="+
          fileStatus.getSourceSURL().toString(),silent);
       util.printMessage("TargetSURL="+
          fileStatus.getTargetSURL().toString(),silent);
       util.printMessage("Status="+
          fileStatus.getStatus().getStatusCode(),silent);
       util.printMessage("Explanation="+
          fileStatus.getStatus().getExplanation(),silent);
       util.printMessage("SRM-CLIENT: ............",silent);
     }
   }

   return;
}


}

