/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2010, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.client.wsdl;
 
import gov.lbl.srm.StorageResourceManager.*;
//import gov.lbl.srm.util.*;
import gov.lbl.srm.client.api.*;
import org.globus.axis.util.Util;

import java.io.*;
import java.net.URL;
import java.util.Vector;
import java.util.HashMap;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;


import javax.xml.rpc.Stub;
import org.apache.axis.types.URI;
import org.apache.axis.types.UnsignedLong;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.*;
import org.apache.log4j.PropertyConfigurator;

//import org.globus.axis.gsi.GSIConstants;
import org.gridforum.jgss.ExtendedGSSCredential;
import org.ietf.jgss.GSSCredential;
import org.gridforum.jgss.ExtendedGSSManager;

//import org.globus.ogsa.utils.GSIUtils;
import org.globus.util.ConfigUtil;
//import org.globus.ogsa.utils.GetOpts;
//import org.globus.ogsa.gui.ServiceContext;
import org.globus.gsi.GlobusCredential;

import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.gsi.gssapi.auth.*;

import org.apache.axis.types.*;
import org.apache.axis.configuration.SimpleProvider;
import org.apache.axis.utils.Options;
import org.apache.axis.SimpleTargetedChain;
import org.globus.axis.transport.*;
import org.apache.axis.transport.http.HTTPSender;
import org.globus.axis.util.Util;

import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;

import gov.lbl.srm.client.exception.*;
import gov.lbl.srm.client.util.*;
import java.util.*;
import java.util.logging.*;

//import org.globus.wsrf.impl.security.authentication.Constants;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMServer
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** GSI-based connection is established and ping the BeStMan for version
    info and its state info. This may happen only once when the 
    web portal starts up.
    SRMServer class is responsible for establishing and maintaining
    connections with the SRM Server.
*/

public class SRMServer {
  private Stub _stub;
  private ISRM _srm;
  private GSSCredential _credential;
  private static boolean _debug;
  private static boolean silent;
  private java.util.logging.Logger _theLogger = 
    java.util.logging.Logger.getLogger
            (gov.lbl.srm.client.wsdl.SRMServer.class.getName());
  private Vector inputVec = new Vector ();
  private String serverUrl;
  private String log4jlocation="";
  private int proxyType;
  private PingStatus pingStatus = new PingStatus(); 

  private java.util.logging.FileHandler _fh;
  private boolean delegationNeeded=false;

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMServer --- constructor
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** constructor
   @param log4jlocation   Location where the log4j.properties  exists
   @param logPath         path where log files can go.  when logpath is providedthe output from the console will go into the logfiles
   @param debug           true or false
*/

public SRMServer (String log4jlocation, String logPath, 
	boolean debug,boolean delegationNeeded) throws SRMClientException 
{
    this._debug = debug;
    if(logPath != null && !logPath.equals("")) {
      silent = true;
    }
    else {
      logPath = null;
    }
    setLogger(logPath,log4jlocation);
    _credential = getCredential("");
    this.delegationNeeded = delegationNeeded;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMServer --- constructor
// User defined proxy file and logDirLocation
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** constructor
   @param proxyFile       Location where proxy file exists
   @param log4jlocation   Location where the log4j.properties  exists
   @param logPath         path where log files can go.  when logpath is providedthe output from the console will go into the logfiles
   @param debug           true or false
*/

public SRMServer (String proxyFile, String logPath, String log4jlocation, 
	boolean debug, boolean delegationNeeded) throws SRMClientException 
{
    if(logPath != null && !logPath.equals("")) {
      silent = true;
    }
    setLogger(logPath, log4jlocation);
    _credential = getCredential(proxyFile);
    this.delegationNeeded = delegationNeeded;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getDebug
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean getDebug() {
  return _debug;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getSilent
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean getSilent() {
  return silent;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setLogger
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void setLogger(String logPath, String log4jlocation) {
   this.log4jlocation = log4jlocation;
   try {
     PropertyConfigurator.configure(log4jlocation);
   }catch(Exception ee){}
   if(silent) {
     //String detailedLogDate=getDetailedLogDate();

     String eventLogPath="";
     if(logPath.endsWith("/")) {
       logPath = logPath.substring(0,logPath.length()-1);
     }
   
     int idx = logPath.lastIndexOf("/");
     String temp = logPath;
     if(idx != -1) {
        temp = logPath.substring(0,idx); 
        File f = new File(temp); 
        if(!f.exists() || !f.isDirectory()) { 
          System.out.println("\nLogPath parent directory does not exists." + temp);
          System.exit(1);
        }
     }

     eventLogPath=logPath;

     try {
      _fh = new java.util.logging.FileHandler(eventLogPath);
      _fh.setFormatter(new NetLoggerFormatter());
      _theLogger.addHandler(_fh);
      _theLogger.setLevel(java.util.logging.Level.ALL);

      File f = new File(eventLogPath+".lck");
      if(f.exists()) {
        f.delete();
      }
      //If this lines are not set, then the annoying globus debug outputs appear on the console.

     }catch(Exception e) {
      System.out.println("Exception="+e.getMessage());
      e.printStackTrace();
     }
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getDetailedLogDate
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private String getDetailedLogDate() {
   Date tDate = new Date();
   Random random = new Random();
   String detailedLogDate = (tDate.getMonth()+1)+"-"+tDate.getDate()+
            "-"+tDate.getHours()+"-"+tDate.getMinutes()+
            "-"+tDate.getSeconds();
   return detailedLogDate+"-"+random.nextInt((int)4.294967296E9);
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getCredential
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private GSSCredential getCredential(String proxyFile) throws SRMClientException {
 GSSCredential mycred = null;
 try {

   if(proxyFile.equals("")) {
    Properties props = System.getProperties();
    Enumeration ee = props.propertyNames();
    while (ee.hasMoreElements()) {
     String str = (String) ee.nextElement();
     if(str.trim().equals("X509_USER_PROXY")) {
       String ttemp = props.getProperty(str.trim());
       if(ttemp != null) {
         proxyFile=ttemp;
       }
     }
    }//end while
   }

   if(proxyFile.equals("")) { //default proxy
      proxyFile  = "/tmp/x509up_u"+MyConfigUtil.getUID();
   }
   inputVec = new Vector ();
   inputVec.addElement("ProxyPath="+proxyFile);
   util.printEventLog(_theLogger,"getCredential",inputVec,silent);
   File f = new File(proxyFile);
   byte[] data = new byte[(int)f.length()];
   FileInputStream in = new FileInputStream (f);
   //read in the credential data
   in.read(data);
   in.close();
   ExtendedGSSManager manager =
    (ExtendedGSSManager) ExtendedGSSManager.getInstance();
   mycred = manager.createCredential(data,
     ExtendedGSSCredential.IMPEXP_OPAQUE,
     GSSCredential.DEFAULT_LIFETIME,
     null, GSSCredential.INITIATE_AND_ACCEPT);
   inputVec = new Vector ();
   inputVec.addElement("CredentialCreated");
   util.printEventLog(_theLogger,"getCredential",inputVec,silent);
   int remainingLifeTime = mycred.getRemainingLifetime();
   if(remainingLifeTime == 0) {
     inputVec = new Vector();
     inputVec.addElement("User credential expired, please renew your credentials");
     util.printEventLog(_theLogger,"CheckTimeLeft", inputVec,silent);
     util.printMessage
         ("SRM-CLIENT: User Credential expired, " +
          "please renew" + " your credentials",silent);
     throw new SRMClientException
        ("SRM-CLIENT: User Credential expired, please renew your credentials");
    }
 
    if(remainingLifeTime <= 1800) { //if <= 30 minutes
     inputVec = new Vector();
     inputVec.addElement("User credential has only " +
		remainingLifeTime + " left please renew your credentials");
     util.printEventLog(_theLogger,"CheckTimeLeft", inputVec,silent);
     util.printMessage
         ("SRM-CLIENT: WARNING: User Credential has only , " + remainingLifeTime + " left."+
          "Please renew" + " your credentials",silent);
    }

   //get proxypath
   GlobusCredential creds = new GlobusCredential(proxyFile);
   proxyType =  creds.getProxyType();
        //proxyType == 14 is full
        //proxyType == 11 is limited
        //proxyType == 15 is pre-RFC
        //proxyType==10 is full legacy globus proxy 
   inputVec = new Vector ();
   inputVec.addElement("ProxyTypeFound="+proxyType);
   util.printEventLog(_theLogger,"getCredential",inputVec,silent);

 }catch(FileNotFoundException fne) { 
   inputVec = new Vector ();
   inputVec.addElement("CredentialNotCreated");
   inputVec.addElement(fne.getMessage());
   util.printEventLog(_theLogger,"getCredential",inputVec,silent);
   throw new SRMClientException ("Proxy file " + proxyFile + " does not exist");
 }
 catch(Exception e) {
   inputVec = new Vector ();
   inputVec.addElement("CredentialNotCreated");
   inputVec.addElement(e.getMessage());
   util.printEventLog(_theLogger,"getCredential",inputVec,silent);
   throw new SRMClientException (e.getMessage());
 }
 return mycred;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// connect
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/**  connects to the SRM server, the service path provided in the serverUrl
*        it creates an SRM server object.    
*     @param serverUrl  -- the serverUrl for the SRM server to be connected
*     @return boolean -- upon successful connection it returns true else false
*/

public boolean connect (String serverUrl) throws SRMClientException, Exception {
	// checking if httpg/https is handled.
    // if not java.net.URL returns exception on httpg.
    // it has been taken care in SRMClientGUI (Util.registerTransport)

    this.serverUrl = serverUrl;
    MyGlobusURL gurl = new MyGlobusURL(serverUrl,0);
    String protocol = gurl.getProtocol();
    String host = gurl.getHost();
    int port = gurl.getPort();
    String path = gurl.getPath();
    int idx = path.indexOf("?SFN");
    if(idx != -1) {
      path = path.substring(0,idx);
    }

    serverUrl = "httpg://"+host+":"+port+path;
    util.printMessage("Connecting to serviceurl : " + serverUrl,silent);
    
    //SRMServiceGridLocator locator = new SRMServiceGridLocator ();
    URL uu = null;
    //gurl = new MyGlobusURL(serverUrl,0);
    //host = gurl.getHost();
    //port = gurl.getPort();
    //path = gurl.getPath();
    SimpleProvider provider = new SimpleProvider();
    SimpleTargetedChain c = null;
    if(serverUrl.startsWith("httpg")) {
       String protocols0 = System.getProperty("java.protocol.handler.pkgs");
       //System.out.println("protocol pkg handler: " + protocols0);
       //System.out.println("....supporting : httpg");
       org.globus.net.protocol.httpg.Handler handler = 
         new org.globus.net.protocol.httpg.Handler();
       try {
         uu = new URL("httpg",host,port,path,handler);
       }catch(Exception h) {
         System.out.print("does not work");
       }
       //System.out.println(" contacting fixed host: " + uu.toString());
       c = new SimpleTargetedChain(new GSIHTTPSender());
       provider.deployTransport("httpg",c);
    }
    else if(serverUrl.startsWith("https")) {
       //System.out.println("....supporting : https");
       c = new SimpleTargetedChain(new HTTPSSender());
       provider.deployTransport("https",c);
       uu = new URL(serverUrl);
    }
    else {
       //System.out.println("....supporting : http");
       c = new SimpleTargetedChain(new HTTPSender());
       provider.deployTransport("http",c);
       uu = new URL(serverUrl);
    }
    Util.registerTransport();
    SRMServiceLocator service = new SRMServiceLocator(provider); 

    ISRM srm = service.getsrm(uu);

    if(srm == null) {
      inputVec = new Vector ();
      inputVec.addElement("remote srm object is null");
      inputVec.addElement("ServiceUrl="+serverUrl);
      util.printEventLog(_theLogger,"SrmUtil-Constructor",inputVec,silent);
      throw new Exception("\nremote srm object is null" + serverUrl);
    }
    else {
      inputVec = new Vector ();
      inputVec.addElement("got remote srm object");
      inputVec.addElement("ServiceUrl="+serverUrl);
      util.printEventLog(_theLogger,"SrmUtil-Constructor",inputVec,silent);
      util.printMessage("\nSRM-CLIENT: got remote srm object",silent);
    }

    _srm = srm;
    _stub = (Stub) srm;
    //opts.setOptions (((Stub)srm));


    //useCredential(_credential);
    if(delegationNeeded) {
	  setSecurityPropertiesForcefully(new URL(serverUrl));
    }
    else {
	  setSecurityProperties(new URL(serverUrl));
    }
    if(srm != null) return true;
    return false;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// disconnect
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** disconnect from the connected SRM server
*/

public void disconnect() {
   if(_srm != null) {
   _srm = null; 
   inputVec = new Vector ();
   inputVec.addElement("ServiceUrl="+serverUrl);
   util.printEventLog(_theLogger,"disconnect",inputVec,silent);
   util.printMessage("\nSRM-CLIENT: disconnected successfully",silent);
  }
  else {
   util.printMessage("\nSRM-CLIENT: SRM object is not connected yet", silent);
   inputVec = new Vector ();
   inputVec.addElement("SRM object is not connected yet");
   inputVec.addElement("ServiceUrl="+serverUrl);
   util.printEventLog(_theLogger,"disconnect",inputVec,silent);
  }
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// useCredential
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
/*
private void useCredential(GSSCredential cred) {
  _stub._setProperty(GSIConstants.GSI_CREDENTIALS, cred);
}
*/

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getEndPoint
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public String getEndPoint() {
  return serverUrl;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setSecurityPropertiesForcefully
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public void setSecurityPropertiesForcefully(URL endpoint) {
    if (endpoint.getProtocol().equals("httpg") ||
         _srm instanceof org.apache.axis.client.Stub) {
       org.apache.axis.client.Stub srm_stub =
                  (org.apache.axis.client.Stub)_srm;
       srm_stub._setProperty(org.globus.axis.gsi.GSIConstants.GSI_CREDENTIALS,
                   _credential);
       srm_stub._setProperty(
         org.globus.axis.transport.GSIHTTPTransport.GSI_AUTHORIZATION,
         org.globus.gsi.gssapi.auth.NoAuthorization.getInstance());
       if(_debug) {
         System.out.println("\nSRM-CLIENT: ProxyType found " + proxyType);
       }
       //GSIUtils.setDelegationGSIProperties(_stub, endpoint);
       if(proxyType == 14 || proxyType == 10) {
        if(_debug) {
          System.out.println("\nSRM-CLIENT: full delegation by client");
        }
        srm_stub._setProperty
        (org.globus.axis.transport.GSIHTTPTransport.GSI_MODE,
         org.globus.axis.transport.GSIHTTPTransport.GSI_MODE_FULL_DELEG);
       }
       else {
        if(_debug) {
           System.out.println("\nSRM-CLIENT: limited delegation by client");
        }
        srm_stub._setProperty
        (org.globus.axis.transport.GSIHTTPTransport.GSI_MODE,
         org.globus.axis.transport.GSIHTTPTransport.GSI_MODE_LIMITED_DELEG);
       }
       srm_stub._setProperty
        (org.globus.gsi.GSIConstants.AUTHZ_REQUIRED_WITH_DELEGATION,
         Boolean.FALSE);
     }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setSecurityProperties
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
private void setSecurityProperties(URL endpoint) {

   if (endpoint.getProtocol().startsWith("httpg") ||
         _srm instanceof org.apache.axis.client.Stub) {
       System.out.println("setting credentials");
       org.apache.axis.client.Stub srm_stub =
              (org.apache.axis.client.Stub)_srm;
       srm_stub._setProperty(
          org.globus.axis.transport.GSIHTTPTransport.GSI_AUTHORIZATION,
          org.globus.gsi.gssapi.auth.NoAuthorization.getInstance());

       if(_debug) {
         System.out.println("\nSRM-CLIENT: ProxyType found " + proxyType);
       }

       //proxyType == 10 is full (old type)
       //proxyType == 14 is full
       //proxyType == 11 is limited
       //proxyType == 15 is pre-RFC
       
        //if(!storageInfo.equals("") || targetHasGsiFTP) {
          //GSIUtils.setDelegationGSIProperties(_stub, endpoint);
          //if(proxyType == 14 || proxyType == 10) {
            //if(_debug) {
              //System.out.println("\nSRM-CLIENT: full delegation by system");
            //}
            //srm_stub._setProperty
             //(org.globus.axis.transport.GSIHTTPTransport.GSI_MODE,
              //org.globus.axis.transport.GSIHTTPTransport.GSI_MODE_FULL_DELEG);
         //}
         //else {
           //if(_debug) {
             //System.out.println("\nSRM-CLIENT: limited delegation by system");
           //}
           //srm_stub._setProperty
           //(org.globus.axis.transport.GSIHTTPTransport.GSI_MODE,
           //org.globus.axis.transport.GSIHTTPTransport.GSI_MODE_LIMITED_DELEG);
         //}
       //}
       //else {
         if(_debug) {
            System.out.println("\nSRM-CLIENT: no delegation by system");
         }
         if(proxyType == 14 || proxyType == 10) {; }
         else {; }
       //}
       srm_stub._setProperty
        (org.globus.gsi.GSIConstants.AUTHZ_REQUIRED_WITH_DELEGATION,
           Boolean.FALSE);
    }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getStub
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public Stub getStub() {
  return _stub;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getISRM
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public ISRM getISRM() {
  return _srm;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getCredential
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public GSSCredential getCredential() {
  return _credential;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getLogger
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public java.util.logging.Logger getLogger() {
  return _theLogger;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getLog4jLocation
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public String getLog4jLocation() {
  return log4jlocation;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// ping
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Check the status of the SRM server. Returns SRM server version info and other info
    @param uid  Authorization Id
    @return boolean returns true after successful ping, else it returns false
*/

public boolean ping(String uid) throws SRMClientException,Exception {
    SRMUtilClient utilClient = new SRMUtilClient(_srm, _theLogger,_debug,silent);
    SrmPingResponse srmPingResponse = utilClient.doPing(uid);

    if(srmPingResponse == null) return false;

    if(srmPingResponse.getVersionInfo() != null) {
      pingStatus.setVersionInfo(srmPingResponse.getVersionInfo());
      pingStatus.setStatus("SRM_SUCCESS");
    }
    else {
      pingStatus.setStatus("SRM_FAILURE");
    }
    ArrayOfTExtraInfo aTExtraInfo = srmPingResponse.getOtherInfo();
    if(aTExtraInfo != null) {
      TExtraInfo[] infos = aTExtraInfo.getExtraInfoArray();
      HashMap map = new HashMap();
      for(int i = 0; i < infos.length; i++) {
        TExtraInfo info = infos[i];
        String key = info.getKey();  
        String value = info.getValue();  
        map.put(key,value);
      }//end for
      pingStatus.setInfo(map);
    }//end if
    return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getVersionInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Returns the version Information of the SRM Server
   @return String  Verson information of the SRM server
*/
public String getVersionInfo() {
  return pingStatus.getVersionInfo();
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getStatus
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Returns SRM_SUCCESS when the ping command is successful. Otherwise returns SRM_FAILURE
   @return String SRM_SUCCESS when successfull else SRM_FAILURE 
   @see String
*/
public String getStatus() {
  return pingStatus.getStatus();
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// findInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** For the given key, it returns the matching value from the extra info provided by the SRM Server
   @param key  key for the extra info
   @return String returns the value for the key provided
   @see String
*/
public String findInfo(String key) throws SRMClientException {
   HashMap map = pingStatus.getInfo();
   Object obj = map.get(key);
   if(obj == null) {
     throw new SRMClientException("Given Key is not found " + key);
   }
   
   String value = (String) obj;
   return value; 
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// printInfo
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** prints the information from the map returned by the ping call
*/

public void printInfo() {
   HashMap map = pingStatus.getInfo();
   Set set = map.entrySet();   
   Iterator i = set.iterator();

   while(i.hasNext()){
    Map.Entry me = (Map.Entry)i.next();
    System.out.println(me.getKey() + " : " + me.getValue() );
   }
}

}
